import logging
import requests
from typing import Dict

from confluence.exceptions.generalerror import ConfluenceError

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ConfluenceVersionConflict(ConfluenceError):
    """Corresponds to 409 errors on the REST API."""

    def __init__(self, path, params, response):
        # type: (str, Dict[str, str], requests.Response) -> None
        msg = 'The given version does not match the expected next version. This is likely because someone else has ' \
              'made a change to the resource at path {}'.format(path)
        super(ConfluenceVersionConflict, self).__init__(path, params, response, msg)



import logging
from enum import Enum
from typing import Any, Dict

from confluence.models.contentbody import ContentBody
from confluence.models.contenthistory import ContentHistory
from confluence.models.space import Space
from confluence.models.version import Version

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ContentType(Enum):
    """
    The set of valid content types in confluence along with their representation on the API.

    https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/ContentType.html
    """

    ATTACHMENT = "attachment"
    BLOG_POST = "blogpost"
    COMMENT = "comment"
    PAGE = "page"


class ContentStatus(Enum):
    """
    The set of valid content statuses in confluence along with their API representation.

    https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/ContentStatus.html
    """

    CURRENT = "current"
    DRAFT = "draft"
    HISTORICAL = "historical"
    TRASHED = "trashed"


class CommentLocation(Enum):
    """The set of valid comment locations as per the Confluence API."""

    INLINE = 'inline'
    FOOTER = 'footer'
    RESOLVED = 'resolved'


class CommentDepth(Enum):
    """The set of depths at which comments can be retrieved over the API."""

    ROOT = ''
    ALL = 'all'


class Content:
    """
    Main content class for all the different content types. This includes pages, blogs, comments and attachments.

    The type field will allow the end user to distinguish between the types from calling code.
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.id = json['id']  # type: int
        self.title = json['title']  # type: str
        self.status = ContentStatus(json['status'])  # type: ContentStatus
        self.type = ContentType(json['type'])  # type: ContentType

        if 'metadata' in json:
            self.metadata = json['metadata']  # type: Dict[str, Any]

        if 'extensions' in json:
            self.extensions = json['extensions']  # type: Dict[str, Any]

        if 'space' in json:
            self.space = Space(json['space'])

        if 'body' in json:
            self.body = ContentBody(json['body'])

        if 'history' in json:
            self.history = ContentHistory(json['history'])

        if 'version' in json:
            self.version = Version(json['version'])

        if self.type == ContentType.ATTACHMENT:
            self.links = json['_links']  # type: Dict[str, Any]

    def __str__(self):
        return '{} - {}'.format(self.id, self.title)


class ContentProperty:
    """
    Represents a single property attached to a piece of content.

    Corresponds to https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/JsonContentProperty.html
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.key = json['key']  # type: str
        self.value = json['value']  # type: Dict[str, Any]
        if 'version' in json:
            self.version = Version(json['version'])
        if 'content' in json:
            self.content = Content(json['content'])

    def __str__(self):
        return self.key



import logging

import pytest

from confluence.exceptions.generalerror import ConfluenceError
from confluence.exceptions.valuetoolong import ConfluenceValueTooLong
from confluence.models.content import ContentType, ContentStatus
from integration_tests.config import get_confluence_instance

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

c = get_confluence_instance()
space_key = 'PSA'


def setup_module():
    c.create_space(space_key, 'Page Space')


def teardown_module():
    c.delete_space(space_key)


def test_create_orphaned_page():
    page = c.create_content(ContentType.PAGE, 'Test orphaned page', space_key, '<h1>Test</h1>')
    assert page.title == 'Test orphaned page'

    c.delete_content(page.id, ContentStatus.CURRENT)


def test_create_page_with_ancestor():
    parent = c.create_content(ContentType.PAGE, 'Parent page', space_key, 'Parent')

    try:
        child = c.create_content(ContentType.PAGE, 'Child page', space_key, 'Child', parent_content_id=parent.id)
        assert child.title == 'Child page'
        children = c.get_child_pages(parent.id)
        assert len(list(children)) == 1

        c.delete_content(child.id, ContentStatus.CURRENT)
    finally:
        c.delete_content(parent.id, ContentStatus.CURRENT)


def test_get_page_content():
    title = 'Full page'
    content = 'This is a full piece of content'
    page = c.create_content(ContentType.PAGE, title, space_key, content=content)
    page = c.get_content_by_id(page.id,
                               expand=['body.storage', 'body.editor', 'body.view', 'body.export_view',
                                       'body.styled_view', 'body.anonymous_export_view'])
    assert page.body.anonymous_export_view == content
    assert hasattr(page.body, 'anonymous_export_view_representation')
    assert page.body.editor == content
    assert hasattr(page.body, 'editor_representation')
    assert page.body.export_view == content
    assert hasattr(page.body, 'export_view_representation')
    assert page.body.storage == content
    assert hasattr(page.body, 'storage_representation')
    assert content in page.body.styled_view
    assert hasattr(page.body, 'styled_view_representation')
    assert page.body.view == content
    assert hasattr(page.body, 'view_representation')

    c.delete_content(page.id, ContentStatus.CURRENT)


def test_create_content_wrong_type():
    with pytest.raises(ValueError):
        c.create_content(ContentType.ATTACHMENT, space_key=space_key, content='', title='')

    with pytest.raises(ValueError):
        c.create_content(ContentType.COMMENT, space_key=space_key, content='', title='')


def test_create_too_large_page():
    with pytest.raises(ConfluenceValueTooLong):
        c.create_content(ContentType.PAGE, space_key=space_key, content='a'*10000000, title='too long')


def test_get_page_more_than_25_results():
    c.create_space('LOTS', 'Lots')
    try:
        for i in range(50):
            c.create_content(ContentType.PAGE, str(i), 'LOTS', content=str(i), expand=['version'])

        pages = list(c.get_content(ContentType.PAGE, space_key='LOTS', expand=['version']))
        assert len(pages) == 51  # NOTE: This relies on no other pages being added to the space and not deleted.
    finally:
        c.delete_space('LOTS')


def test_update_page_content():
    # Create test page
    title = 'Full page updated'
    content = 'This is a full piece of content'
    result = c.create_content(ContentType.PAGE, title, space_key, content=content, expand=['body.storage', 'version'])
    assert result.body.storage == content

    # Update test page
    new_content = 'This is updated content'
    new_title = 'Updated title'
    result = c.update_content(result.id, result.type, result.version.number + 1, new_content, new_title)
    assert result.title == new_title
    assert result.body.storage == new_content

    # Read updated page
    result = c.get_content_by_id(result.id, expand=['body.storage'])
    assert result.title == new_title
    assert result.body.storage == new_content

    c.delete_content(result.id, ContentStatus.CURRENT)


def test_minor_update():
    result = c.create_content(ContentType.PAGE, 'Minor Doc', space_key, content='Hello')
    update_result = c.update_content(result.id, result.type, result.version.number + 1, 'Hello minor', 'Minor Doc',
                                     minor_edit=True, edit_message='An edit message', expand=['version'])
    assert update_result.version.message == 'An edit message'
    # Note that the API says it wasn't a minor edit but that's a lie so commenting this check out.
    # assert update_result.version.minor_edit

    c.delete_content(update_result.id, ContentStatus.CURRENT)


def test_get_content_no_results():
    result = list(c.get_content(ContentType.PAGE, space_key=space_key, title='Nothing here'))
    assert len(result) == 0


def test_create_duplicate_page():
    page = c.create_content(ContentType.PAGE, 'Duplicate Page', space_key, '1')

    with pytest.raises(ConfluenceError):
        c.create_content(ContentType.PAGE, 'Duplicate Page', space_key, '1')

    c.delete_content(page.id, ContentStatus.CURRENT)


def test_create_page_in_nonexistent_space():
    with pytest.raises(ConfluenceError):
        c.create_content(ContentType.PAGE, 'Bad page', 'NONSENSE', 'Test')


def test_get_content_with_bad_content_type():
    with pytest.raises(ValueError):
        c.get_content(ContentType.ATTACHMENT)



import logging

import pytest

from confluence.exceptions.resourcenotfound import ConfluenceResourceNotFound
from confluence.models.content import ContentType, ContentStatus
from integration_tests.config import get_confluence_instance

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

c = get_confluence_instance()
space_key = 'WOS'


def setup_module():
    c.create_space(space_key, 'Watch Space')


def teardown_module():
    c.delete_space(space_key)


def test_user_not_watching_content():
    page = c.create_content(ContentType.PAGE, space_key=space_key, title='Test', content='Test')
    try:
        c.remove_content_watch(page.id)  # Remove the watch because we created the content so it's watched by default!
        assert not c.is_user_watching_content(page.id)
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)


def test_user_watching_content():
    page = c.create_content(ContentType.PAGE, space_key=space_key, title='Test', content='Test')
    try:
        c.add_content_watch(page.id)
        assert c.is_user_watching_content(page.id)
        c.remove_content_watch(page.id)
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)


def test_user_is_watching_content_by_username():
    page = c.create_content(ContentType.PAGE, space_key=space_key, title='Test', content='Test')
    try:
        c.add_content_watch(page.id, username='admin')
        assert c.is_user_watching_content(page.id, username='admin')
        c.remove_content_watch(page.id, username='admin')
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)


def test_user_is_watching_content_by_key():
    page = c.create_content(ContentType.PAGE, space_key=space_key, title='Test', content='Test')
    try:
        user = c.get_user(username='admin')
        c.add_content_watch(page.id, user_key=user.user_key)
        assert c.is_user_watching_content(page.id, user_key=user.user_key)
        c.remove_content_watch(page.id, user_key=user.user_key)
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)


def test_bad_user_of_content_watch_functions():
    with pytest.raises(ValueError):
        c.add_content_watch(1, "a", "a")
    with pytest.raises(ValueError):
        c.is_user_watching_content(1, "a", "a")
    with pytest.raises(ValueError):
        c.remove_content_watch(1, "a", "a")


def test_user_not_watching_space():
    assert not c.is_user_watching_space(space_key, username='admin')


def test_user_is_watching_space():
    c.add_space_watch(space_key, username='admin')
    assert c.is_user_watching_space(space_key, username='admin')
    c.remove_space_watch(space_key, username='admin')


def test_user_is_watching_space_by_key():
    user = c.get_user(username='admin')
    c.add_space_watch(space_key, user_key=user.user_key)
    assert c.is_user_watching_space(space_key, user_key=user.user_key)
    c.remove_space_watch(space_key, user_key=user.user_key)


def test_remove_space_watch_without_one():
    c.remove_space_watch(space_key, username='admin')
    assert True


def test_current_user_watching_space():
    c.add_space_watch(space_key)
    assert c.is_user_watching_space(space_key)
    c.remove_space_watch(space_key)


def test_watching_non_existent_space():
    with pytest.raises(ConfluenceResourceNotFound):
        c.add_space_watch("NONSENSE")
    with pytest.raises(ConfluenceResourceNotFound):
        c.is_user_watching_space("NONSENSE")
    with pytest.raises(ConfluenceResourceNotFound):
        c.remove_space_watch("NONSENSE")


def test_bad_user_of_space_watch_functions():
    with pytest.raises(ValueError):
        c.add_space_watch(space_key, "a", "a")
    with pytest.raises(ValueError):
        c.is_user_watching_space(space_key, "a", "a")
    with pytest.raises(ValueError):
        c.remove_space_watch(space_key, "a", "a")



from confluence.exceptions.resourcenotfound import ConfluenceResourceNotFound
from confluence.models.content import ContentType, ContentStatus
from integration_tests.config import get_confluence_instance
import logging
import py
import pytest

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

c = get_confluence_instance()
space_key = 'ATTSP'


def setup_module():
    c.create_space(space_key, 'Page Space')


def teardown_module():
    c.delete_space(space_key)


def test_add_attachment(tmpdir):  # type: (py.path.local) -> None
    page_id = c.create_content(ContentType.PAGE, space_key=space_key, content='', title='Test Attachment Page').id

    try:
        p = tmpdir.mkdir("attachments").join("test.txt")
        p.write("test")
        c.add_attachment(page_id, p.realpath())
        attachments = list(c.get_attachments(page_id, filename=None, media_type=None))
        assert len(attachments) == 1
    finally:
        c.delete_content(page_id, ContentStatus.CURRENT)


def test_get_attachment_by_filename(tmpdir):  # type: (py.path.local) -> None
    page_id = c.create_content(ContentType.PAGE, space_key=space_key, content='', title='Test Filename Attachment Page').id

    try:
        p = tmpdir.mkdir("attachments").join("test.txt")
        p.write("test")
        c.add_attachment(page_id, p.realpath())
        c.add_attachment(page_id, p.realpath(), file_name='other_file.txt')
        attachments = list(c.get_attachments(page_id, filename='test.txt', media_type=None))
        assert len(attachments) == 1
    finally:
        c.delete_content(page_id, ContentStatus.CURRENT)


def test_add_attachment_to_missing_page(tmpdir):  # type: (py.path.local) -> None
    p = tmpdir.mkdir("attachments").join("bad.txt")
    p.write("bad")
    with pytest.raises(ConfluenceResourceNotFound):
        c.add_attachment(-1, file_path=p.realpath())


def test_download_attachment(tmpdir):  # type: (py.path.local) -> None
    page_id = c.create_content(ContentType.PAGE, space_key=space_key, content='', title='Test Download Attachment Page').id

    try:
        p = tmpdir.mkdir("attachments").join("test.txt")
        p.write("test")
        attachments = list(c.add_attachment(page_id, p.realpath()))
        file_contents = c.download_attachment(attachments[0])
        assert file_contents == b"test"
    finally:
        c.delete_content(page_id, ContentStatus.CURRENT)



import logging

from confluence.models.content import ContentType, ContentStatus

from integration_tests.config import get_confluence_instance

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

c = get_confluence_instance()
space_key = 'CH'


def setup_module():
    c.create_space(space_key, 'Content History Space')


def teardown_module():
    c.delete_space(space_key)


def test_get_page_history_with_no_expansion():
    page = c.create_content(ContentType.PAGE, content='A', space_key=space_key, title='A')
    try:
        c.update_content(page.id, ContentType.PAGE, new_version=page.version.number + 1, new_content='AA',
                         new_title=page.title)
        history = c.get_content_history(page.id)
        assert history.author.username == 'admin'
        assert history.last_updated.number == 2
        assert history.previous_version.number == 1
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)


def test_get_page_history_with_all_expansions():
    page = c.create_content(ContentType.PAGE, content='A', space_key=space_key, title='A')
    try:
        c.update_content(page.id, ContentType.PAGE, new_version=page.version.number + 1, new_content='AA',
                         new_title=page.title)
        history = c.get_content_history(page.id, expand=['previousVersion', 'nextVersion', 'lastUpdated',
                                                         'contributors.publishers'])
        assert history.author.username == 'admin'
        assert history.last_updated.number == 2
        assert history.previous_version.number == 1
        assert history.contributors is not None
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)



import logging

from confluence.models.content import ContentType, ContentStatus
from confluence.models.label import LabelPrefix
from integration_tests.config import get_confluence_instance

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

c = get_confluence_instance()
space_key = 'LO'


def setup_module():
    c.create_space(space_key, 'Label Operations Space')


def teardown_module():
    c.delete_space(space_key)


def test_add_get_delete_label():
    page = c.create_content(ContentType.PAGE, content='A', space_key=space_key, title='A')
    try:
        c.create_labels(page.id, [(LabelPrefix.GLOBAL, 'b'), (LabelPrefix.MY, 'd')])

        labels = list(c.get_labels(page.id))
        prefixed_labels = list(c.get_labels(page.id, LabelPrefix.GLOBAL))
        assert len(labels) == 2
        assert len(prefixed_labels) == 1

        c.delete_label(page.id, labels[0].name)
    finally:
        c.delete_content(page.id, ContentStatus.CURRENT)



import logging
from enum import Enum
from typing import Any, Dict

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class LabelPrefix(Enum):
    """
    Represents the valid prefix values for a Label.

    c.f. https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/api/model/content/Label.Prefix.html
    """

    GLOBAL = 'global'
    MY = 'my'
    SYSTEM = 'system'
    TEAM = 'team'


class Label:
    """
    Represents a label on a piece of confluence content.

    c.f. https://docs.atlassian.com/atlassian-confluence/6.6.0/com/atlassian/confluence/labels/Label.html
    """

    def __init__(self, json):  # type: (Dict[str, Any]) -> None
        self.id = json['id']  # type: str
        self.name = json['name']  # type: str
        self.prefix = json['prefix']  # type: str

    def __str__(self):
        return self.name



import logging

from confluence.models.content import ContentProperty

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


def create_with_minimal_json():
    p = ContentProperty({
        'key': 'KEY',
        'value': {}
    })

    assert str(p) == 'KEY'
    assert not hasattr(p, 'content')
    assert not hasattr(p, 'version')


def create_with_full_json():
    p = ContentProperty({
        'key': 'KEY',
        'value': {
            'anything': 1
        },
        'version': {
            'number': 2,
            'minorEdit': False,
            'hidden': False
        },
        'content': {
            'id': 1,
            'title': 'Hello',
            'status': 'current',
            'type': 'page'
        }
    })

    assert str(p) == 'KEY'
    assert p.version.number == 2
    assert p.content.id == 1



import logging
import requests
from typing import Dict

from confluence.exceptions.generalerror import ConfluenceError

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class ConfluenceAuthenticationError(ConfluenceError):
    """This exception corresponds to 401 errors on the REST API."""

    def __init__(self, path, params, response):
        # type: (str, Dict[str, str], requests.Response) -> None
        msg = 'Authentication failure. This is most likely due to incorrect username/password'.format(path)
        super(ConfluenceAuthenticationError, self).__init__(path, params, response, msg)



import pytest

from confluence.client import Confluence
from confluence.exceptions.authenticationerror import ConfluenceAuthenticationError
from integration_tests.config import local_url


def test_bad_username_password():  # type: () -> None
    c = Confluence(local_url, ('bad', 'bad'))
    with pytest.raises(ConfluenceAuthenticationError):
        c.create_space('OK', 'ok')
